#

from buildz.tz import xfind as sh
from buildz import xf
s = r"""
name=zero
age=12000
datas = [
    {
        name=notebook
        price=6500
        power=200w
        contains=[buildz,pyz]
    }
    {
        name=buildz
        function='easy coding'
        contains = [
            {
                name=xf
                function = 'json-like easier format'
            }
            {
                name=ioc
                function = 'factory build items by profile or wrap'
            }
            {
                name=auto
                function = 'easy to make auto functions'
            }
        ]
    }
    {
        name=pyz
        function = 'tools'
        contains = [
            {
                name=nxf
                function = 'safely transfer data by devices'
            }
            {
                name=netx
                function = 'both side RPC support'
            }
            {
                name=logz
                function = 'log recoding'
            }
        ]
    }
]
"""
"""

python test_xfind.py

"""
data = xf.loads(s)
print(xf.dumps(data, format=1))
node = sh.Node.parse(data)
arr = sh.SearchArray.single(node)
pts = r"""
[
    [
        or, 
        (mkv, name, nxf)
        (mkv, name, xf)
    ]
    [re, key, "[f].*"]
]
"""
arr = arr.searchs_dfc(pts)
print("Rst:", arr.datas())
print("Vals:", arr.vals())



#coding=utf-8
import os
"""
读写文件再简化
"""
def fread(fp, mode='rb'):
    with open(fp, mode) as f:
        return f.read()

pass
read=fread
def freads(fp, mode = 'rb', size=1024*1024):
    with open(fp, mode) as f:
        while True:
            bs = f.read(size)
            if len(bs)==0:
                break
            yield bs

pass
reads=freads

def fwrite(ct, fp, mode = 'wb'):
    with open(fp, mode) as f:
        f.write(ct)

pass
write = fwrite
def fwrites(cts, fp, mode = 'wb'):
    with open(fp, mode) as f:
        for ct in cts:
            f.write(ct)

pass
writes = fwrites

def makedir(dp):
    if os.path.isdir(dp):
        return
    os.makedirs(dp)

pass
def makefdir(fp):
    fp = os.path.abspath(fp)
    dp = os.path.dirname(fp)
    makedir(dp)

pass

def dirpath(fp, n=1):
    for i in range(n):
        fp = os.path.dirname(fp)
    return fp

pass

dirname = dirpath

def fcover(filepath, wsize = 1024*10):
	st = os.stat(filepath)
	size = st.st_size
	bs = b'a'*wsize
	with open(filepath, 'wb') as f:
		for i in range(0, size, wsize):
			f.write(bs)

pass
cover = fcover
def removes(fp, cover = False):
    if not os.path.exists(fp):
        return
    if os.path.isfile(fp):
        #print(f"remove file '{fp}'")
        if cover:
            fcover(fp)
        os.remove(fp)
        return
    fps = os.listdir(fp)
    fps = [os.path.join(fp, f) for f in fps]
    [removes(f, cover) for f in fps]
    #print(f"removedirs '{fp}'")
    os.rmdir(fp)

pass
def decode(s, coding = 'utf-8'):
    coding = coding.lower()
    xcoding = 'utf-8'
    if coding == 'utf-8':
        xcoding = 'gbk'
    try:
        return s.decode(coding)
    except:
        return s.decode(xcoding)

pass

def sread(fp, code='utf-8', mode='r'):
    if mode.find("b")<0:
        mode+="b"
    return decode(read(fp, mode), code)

pass
def swrite(dt, fp, code="utf-8", mode = "w"):
    if mode.find("b")<0:
        mode+="b"
    if type(dt)!=bytes:
        dt = dt.encode(code)
    write(dt, fp, mode)

pass

def is_abs(fp):
    if fp is None:
        return False
    if fp.strip()=="":
        return False
    fp = fp.strip().replace("\\", "/")
    if fp[0]=="/":
        return True
    arr = fp.split("/")
    if arr[0].find(":")>=0:
        return True
    return False

pass
    


#coding=utf-8
#from .read import loads
from .readz import loads, loadx
import sys
def fetch(argv = None):
    if argv is None:
        argv = sys.argv[1:]
    argv = [str(k) for k in argv]
    s = " ".join(argv)
    s = s.strip()
    return loadx(s, out_args=True)
    lrs = [["",""]]
    if len(s)==0:
        return None
    if s[0] not in "([{":
        lrs = ["{}", "[]"]
    for l,r in lrs:
        x = l+s+r
        try:
            return loads(x)
        except:
            pass
    raise Exception("unable to fetch params")

pass
# python -m buildz.xf {a:b}
def test():
    rst = fetch()
    print(f"cmd params: {rst}")

pass



#coding=utf-8
from . import base
import termios
import sys
import ctypes
import select
std = ctypes.cdll.LoadLibrary(None)
class Getch(base.Getch):
    def __init__(self, timeout =0.01, cnt = 3):
        attr = termios.tcgetattr(sys.stdin.fileno())
        bak = list(attr)
        attr[3] = attr[3] & ~(termios.ICANON|termios.ECHO)
        self.bak = bak
        self.attr = attr
        self.sec = timeout
        self.cnt = cnt
    def _open(self):
        termios.tcsetattr(sys.stdin.fileno(), termios.TCSANOW, self.attr)
    def _close(self):
        termios.tcsetattr(sys.stdin.fileno(), termios.TCSANOW, self.bak)
    def call(self):
        c = std.getchar()
        if c==27:
            next = False
            # 没用
            for i in range(self.cnt):
                a,_b,_c = select.select([sys.stdin.fileno()], [], [], self.sec)
                if len(a)>0:
                    next = True
                    break
            if next:
                rst = [c]
                rst.append(std.getchar())
                rst.append(std.getchar())
                return rst
        return c

pass



try:
    import pymysql
except ModuleNotFoundError:
    raise Exception("module not found, try: pip install pymysql")
import sys
from .basez import SimpleDv, fetch
from .structz import CMD
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        result = []
        if rst is None or len(rst)==0:
            return result
        a = rst[0]
        keys = list(a.keys())
        result.append(keys)
        for obj in rst:
            v = [obj[k] for k in keys]
            result.append(v)
        return result
    def new_con(self):
        return pymysql.connect(host=self.host, 
            port = self.port, user =self.user, 
            password =self.pwd, database = self.db,
            charset='utf8',init_command="SET SESSION time_zone='+08:00'")
    def new_cursor(self):
        return self.con.cursor(pymysql.cursors.DictCursor)
    def init(self, *argv, **maps):
        if self.port is None:
            self.port = 3306
        pass
    def sql_tables(self, table = None):
        """
            if not note, use name instead
            require:
                table_name, table_note
        """
        query_table = ""
        if table is not None:
            query_table =  f" and table_name = '{table}'"
        return f"select table_name, table_comment as table_note from information_schema.tables where table_schema='{self.db}' {query_table};"
    def sql_columns(self, table):
        """
            require:
                table_name, column_name, column_type, column_default, nullable, column_offset, column_note
        """
        return f"select table_name, column_name, column_type, column_default, is_nullable as nullable, ordinal_position as column_offset, column_comment as column_note from information_schema.columns where table_schema='{self.db}' and table_name = '{table}' order by ordinal_position asc;"
    def sql_indexes(self, table=None):
        """
            require:
                table_name, index_name, is_unique, index_type, index_note
        """
        query_table = ""
        if table is not None:
            query_table =  f" and table_name = '{table}'"
        return f"select table_name, index_name, 1-any_value(non_unique) as is_unique, any_value(index_type) as index_type, any_value(index_comment) as index_note from information_schema.statistics where table_schema='{self.db}' {query_table} group by index_name;"
    def sql_index_keys(self, index):
        """
            require:
                index_name, column_name, index_offset, column_note
        """
        return f"select index_name, column_name, seq_in_index as index_offset, comment as column_note from information_schema.statistics where table_name='flow_review' and table_schema='{self.db}' and index_name = '{index}' order by seq_in_index asc;"

pass
def build(argv, conf):
    dv = Db(*fetch(argv))
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass



#coding=utf-8
from buildz import xf, argx, pyz, ioc, fz

from ..test import text_sfx, args, maps
class Deal:
    def deal(self):
        if len(args)<2:
            print("need params 1 to be dirpath")
            return
        dp = args[1]
        sfx = argx.get(maps, ("s", "suffix"), "js")
        fps = fz.search(dp, f".*\.{sfx}$")
        confs = ioc.build()
        confs.add_fps(fps)
        id = argx.get(maps, ("i", "id"), "main")
        rst = confs.get(id)
        print(f"get {id}: {rst}")
        return rst

pass




'''
数据封装类
val是数据（字符）值
pos是字符在文件里的位置(第几行第几列)
remain和type？
'''
class PrevItem:
    def __str__(self):
        return "<prev_item val={val}, type = {type}, pos = {pos}, maps={maps}>".format(val = str(self.val), type = str(self.type), pos = str(self.pos), maps=self._maps)
    def __repr__(self):
        return str(self)
    def none(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)==v:
                return False
        return True
    def any(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)==v:
                return True
        return False
    def check(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)!=v:
                return False
        return True
    def maps(self, key):
        if key not in self._maps:
            if key == 'type':
                return self.type
            return None
        return self._maps[key]
    def __init__(self, val, pos, type = None, remain=None, **maps):
        self.val= val
        self.pos= pos
        self.remain= remain
        self._maps = maps
        self.type = type

pass
class DealItem(PrevItem):
    def __str__(self):
        return "<deal_item val={val}, type = {type}, pos = {pos}, maps={maps}>".format(val = str(self.val), type = str(self.type), pos = str(self.pos), maps=self._maps)




from .. import base


class PrevSpcDeal(base.BaseDeal):
    def has_prev(self):
        return 1
    def has_deal(self):
        return 0
    """
        去掉左空格
    """
    def prev(self, buffer, queue, pos):
        if buffer.size()>0:
            return False
        c = buffer.read()
        if len(c)==0:
            return False
        if len(c.strip())==0:
            buffer.pop_read()
            buffer.clean()
            return True
        return False

pass



try:
    import clickhouse_driver
except ModuleNotFoundError:
    raise Exception("module not found, try: pip install clickhouse-driver")
from clickhouse_driver import connect
from clickhouse_driver import Client
import sys
from .basez import SimpleDv, fetch
from .structz import CMD
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        rows = self.cursor.description
        keys = [k[0].lower() for k in rows]
        result = []
        result.append(keys)
        if rst is None or len(rst)==0:
            return result
        result += rst
        return result
    def new_con(self):
        return connect('clickhouse://{user}:{pwd}@{host}:{port}/{db}'.format(user = self.user, db = self.db, pwd =self.pwd, host = self.host, port = self.port))
    def new_cursor(self):
        return self.con.cursor()
    def init(self, *argv, **maps):
        pass

pass
def build(argv, conf):
    dv = Db(*fetch(argv))
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass



#


from . import xf, ioc, fz
from .base import Base
from .ioc import wrap
#from .tools import *
import time, sys
class Log(Base):
    def tag(self, _tag):
        log = Log(self.shows, _tag, self)
        return log
    def get_tag(self):
        return self._tag
    def init(self, shows = None, tag= None, base = None):
        if shows is None:
            shows = ["info", "debug", "warn", "error"]
        self.shows=shows
        self._tag = tag
        self.base = base
    def xlog(self, level, *args):
        if level not in self.shows:
            return
        self.log(level, self.tag, *args)
    def log(self, level, tag, *args):
        if self.base is not None:
            return self.base.log(level, tag, *args)
        raise Exception("unimpl")
    def info(self, *args):
        if "info" not in self.shows:
            return
        self.log("info", self._tag, *args)
    def warn(self, *args):
        if "warn" not in self.shows:
            return
        self.log("warn", self._tag, *args)
    def debug(self, *args):
        if "debug" not in self.shows:
            return
        self.log("debug", self._tag, *args)
    def error(self, *args):
        if "error" not in self.shows:
            return
        self.log("error", self._tag, *args)

pass
def replaces(s, *args):
    for i in range(0,len(args),2):
        k,v = args[i],args[i+1]
        s = s.replace(k,v)
    return s

pass
def mstr(s):
    if s is None or len(s)==0:
        return s
    return s[:1].upper()+s[1:].lower()
class FormatLog(Log):
    def init(self, shows =None, tag=None, format=None):
        if format is None:
            format = "[{LEVEL}] %Y-%m-%d %H:%M:%S {tag} {msg}\n"
        self.format=format
        super().init(shows, tag)
    def output(self, msg):
        raise Exception("impl")
    def log(self, level, tag, *args):
        m_level = level.lower()
        u_level = level.upper()
        x_level = mstr(level)
        args = [str(k) for k in args]
        msg = " ".join(args)
        if tag is None:
            tag = "base"
        rst = time.strftime(self.format, time.localtime(time.time()))
        msg = replaces(rst, "{Level}", x_level, "{level}", m_level, "{LEVEL}", u_level, "{tag}", tag, "{msg}", msg)
        self.output(msg)
class FpLog(FormatLog):
    def init(self, fp = None,shows =None, tag=None, format=None):
        super().init(shows, tag, format)
        self.fp = fp
    def output(self, msg):
        sys.stdout.write(msg)
        if self.fp is not None:
            fp = time.strftime(self.fp)
            fz.makefdir(fp)
            fz.write(msg.encode("utf-8"), fp, 'ab')

pass
class StdLog(FormatLog):
    def output(self, msg):
        sys.stdout.write(msg)

pass

class Logs(Log):
    def init(self, logs, shows = None, tag= None):
        super().init(shows, tag)
        self.logs = logs
    def log(self, level, tag, *args):
        for _log in self.logs:
            _log.log(level, tag, *args)

pass

def build(obj=None, shows=None, tag=None, format=None):
    if obj is None:
        return StdLog(shows, tag, format)
    return obj

pass
make=build


#coding=utf-8
from . import pyz
class Base:
    def str(self):
        return str(self.__class__)
    def __str__(self):
        return self.str()
    def __repr__(self):
        return self.__str__()
    def __init__(self, *args, **maps):
        self.init(*args, **maps)
    def __call__(self, *args, **maps):
        return self.call(*args, **maps)
    def init(self, *args, **maps):
        pass
    def call(self, *args, **maps):
        return self.deal(*args, **maps)
    def deal(self, *args, **maps):
        return None

pass

class WBase(Base):
    def _open(self):
        pass
    def open(self):
        self._open()
        return pyz.with_out(self.close, True)
    def _close(self):
        pass
    def close(self, exc_type, exc_val, exc_tb):
        self._close()

pass

class With(Base):
    def init(self, args=False):
        self.args = args
    def call(self, cls):
        def _open(obj):
            pass
        def open(obj):
            obj._open()
            return pyz.with_out(obj.close, self.args)
        def _close(obj):
            pass
        def close(obj):
            obj._close()
        cls._open = _open
        cls._close = _close
        cls.open = open
        cls.close = close
        return cls

pass



#coding=utf-8
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class IOCFObjectDeal(FormatDeal):
    """
        iocf字段ioc:
            {
                id:id
                type: iocf

                filepath: item_conf
                // or
                folder: item_conf
                pattern: item_conf
            }
        简写:
            [[id, ioc], filepath]
        例:
            [iocf, filepath] //
    """
    def init(self, fp_lists=None, fp_defaults=None):
        self.fps = set()
        self.ids = set()
        super().init("IOCFObjectDeal", fp_lists, fp_defaults, join(dp, "conf", "iocf_lists.js"), None)
    def deal(self, edata:EncapeData):
        data = edata.data
        id = xf.g(data, id = None)
        data = self.fill(data)
        if id is not None:
            if id in self.ids:
                return None
            else:
                self.ids.add(id)
        fp = xf.g(data, filepath = None)
        fdir = xf.g(data, folder = None)
        pt = xf.g(data,  pattern = None)
        if fp is None:
            if fdir is None:
                raise Exception("filepath can't be null")
            else:
                fdir = self.get_obj(fdir, edata.conf)
                if pt is not None:
                    pt = self.get_obj(pt, edata.conf)
                fps = fz.search(fdir, pt)
        else:
            fp = self.get_obj(fp, edata.conf)
            fps = [fp]
        fps = [k for k in fps if k not in self.fps]
        if len(fps)==0:
            return None
        edata.conf.confs.add_fps(fps)
        self.fps.update(fps)
        return None

pass



import sys
from .structz import ItDv
from buildz.db import tls
from buildz import xf
def sp(obj):
    return super(obj.__class__, obj)

pass
class SimpleDv(ItDv):
    # func to impl:
    def to_list(self, query_result):
        return []
    def sql_tables(self, table = None):
        """
            if not note, use name instead
            require:
                table_name, table_note
        """
        raise Exception("unimplement")
        return None
    def sql_columns(self, table):
        """
            require:
                table_name, column_name, column_type, column_default, nullable, column_offset, column_note
        """
        raise Exception("unimplement")
        return None
    def sql_indexes(self, table=None):
        """
            require:
                table_name, index_name, is_unique, index_type, index_note
        """
        raise Exception("unimplement")
        return None
    def sql_index_keys(self, index):
        """
            require:
                index_name, column_name, index_offset, column_note
        """
        raise Exception("unimplement")
        return None
    def new_con(self):
        return None
    def new_cursor(self):
        return None
    def init(self, *argv, **maps):
        pass
    def tables(self, table=None, as_map=None):
        sql = self.sql_tables(table)
        return self.query(sql, as_map=as_map)
    def columns(self, table, as_map=None):
        sql = self.sql_columns(table)
        return self.query(sql, as_map=as_map)
    def indexes(self, table=None, as_map=None):
        sql = self.sql_indexes(table)
        return self.query(sql, as_map=as_map)
    def index_keys(self, index, as_map=None):
        sql = self.sql_index_keys(table)
        return self.query(sql, as_map=as_map)
    # func already impl
    def check_query(self, s):
        arr = s.split(" ")
        k = arr[0].strip().lower()
        rst = k not in "delete,insert,update,create,drop,commit".split(",")
        return rst
    def out_list(self, query_result, as_map=None):
        if as_map is None:
            as_map = self.as_map
        rst = self.to_list(query_result)
        if as_map and len(rst)>0:
            if len(rst)==1:
                return []
            keys = rst[0]
            rst = rst[1:]
            rst = [{k:v for k,v in zip(keys, dt)} for dt in rst]
        return rst
    def __init__(self, host, port, user, pwd, db, as_map=False, *argv, **maps):
        self.host = host
        self.port = port
        self.user = user
        self.pwd = pwd
        self.db = db
        self.con = None
        self.cursor = None
        self.as_map = as_map
        self.init(*argv, **maps)
    def begin(self):
        self.con = self.new_con()
        self.cursor = self.new_cursor()
    def open(self):
        self.begin()
    def close(self):
        self.cursor.close()
        self.con.close()
        self.cursor = None
        self.con = None
    def is_open(self):
        return self.cursor is not None
    def commit(self):
        self.cursor.close()
        self.con.commit()
        self.cursor = self.new_cursor()
    def refresh(self):
        self.cursor.close()
        self.cursor = self.new_cursor()
    def query(self, sql, vals=(), as_map = None):
        # return list, first row is key
        tmp = self.cursor.execute(sql, vals)
        #print("[TESTZ] exe:",tmp)
        rst = self.cursor.fetchall()
        return self.out_list(rst, as_map)
    def executes(self, sqls, commit=False):
        if type(sqls)==str:
            sqls = sqls.split(";")
        sqls = [sql.strip() for sql in sqls if sql.strip()!=""]
        _ = [self.execute(sql) for sql in sqls]
        if commit:
            self.execute("commit;")
    def execute(self, sql, vals=(), commit=False):
        tmp = self.cursor.execute(sql, vals)
        if commit:
            self.cursor.execute("commit;")
        return tmp
    def insert_or_update(self, maps, table, keys = None):
        if type(maps)!=dict:
            maps = maps.__dict__
        if keys is None:
            keys = []
        if type(keys) not in (list, tuple):
            keys = [keys]
        update = False
        conds = ""
        if len(keys)>0:
            need_query = True
            conds = []
            for k in keys:
                if k not in maps:
                    need_query = False
                    break
                v = maps[k]
                if type(v)==str:
                    v = f"'{v}'"
                if v is not None:
                    cond = f"{k} = {v}"
                else:
                    cond = f"{k} is null"
                conds.append(cond)
            if need_query:
                conds = " and ".join(conds)
                sql_search = f"select count(*) from {table} where {conds}"
                rst = self.query(sql_search, as_map = False)[1][0]
                update = rst>0
        if update:
            keys = set(keys)
            kvs = [[k,tls.py2sql(v)] for k,v in maps.items() if k not in keys]
            sets = [f"{k}={v}" for k,v in kvs]
            sets = ",".join(sets)
            sql = f"update {table} set {sets} where {conds}"
        else:
            kvs = [[k, tls.py2sql(v)] for k,v in maps.items()]
            ks = ",".join([kv[0] for kv in kvs])
            vs = ",".join([str(kv[1]) for kv in kvs])
            sql = f"insert into {table}({ks}) values({vs});"
        return self.execute(sql)

pass

# user|pwd@host:port/db
def fetch(args, conf={}):
    """
        host[:port][/db][ user][ pwd]
        host:port/db user pwd
    """
    if type(args) == str:
        args = args.split(" ")
    url = args[0].strip()
    tmp = url.split("/")
    url = tmp[0]
    db = None
    if len(tmp)>1:
        db = tmp[1]
    tmp = url.split(":")
    if len(tmp)==2:
        host, port = tmp
        port = int(port)
    else:
        host = tmp[0]
        port = None
    user = None
    if len(args)>1:
        user = args[1]
        if user is not None:
            user = user.strip()
    pwd = None
    if len(args)>2:
        pwd = args[2]
        if pwd is not None:
            pwd = pwd.strip()
    as_map = xf.g(conf, as_map=False)
    return host, port, user, pwd, db, as_map

pass



from .. import base
from .. import itemz
import re
import json
class StrDeal(base.BaseDeal):
    def rpt(self, pt):
        st = self.like("^", pt)
        ed = self.like("$", pt)
        if pt[0]!=st:
            pt = st+pt
        if pt[-1]!=ed:
            pt = pt+ed
        return pt
    def init(self, left, right, pts = []):
        self.left = left
        self.right = right
        #self.single_line = single_line
        self.pts = [self.rpt(pt) for pt in pts]
    def deal(self, obj, conf):
        val = obj.val
        if type(val) not in [bytes, str]:
            return None
        et = self.like("\n", val)
        need_ep = 0
        if len(val)==0:
            need_ep = 1
        elif val.find(et)>=0:
            need_ep = 1
        else:
            for pt in self.pts:
                rpt = self.like(pt, val)
                if re.match(rpt, val) is not None:
                    need_ep = 1
                    break
        if need_ep:
            if type(val)==bytes:
                val = val.decode()
                val = json.dumps(val, ensure_ascii=False)[1:-1]
                val = val.encode()
            else:
                val = json.dumps(val, ensure_ascii=False)[1:-1]
            val = conf.s(self.left)+val+conf.s(self.right)
        return itemz.ShowItem(val, obj.deep, is_val=1, is_done=1)

pass 



            

                


#coding=utf-8

maps = {
    '"': '"', '\\': '\\', '/': '/',
    'b': '\b', 'f': '\f', 'n': '\n', 'r': '\r', 't': '\t',
}
bmaps = {k.encode()[0]:v.encode() for k,v in maps.items()}
def ustr(s):
    i = 0
    l=len(s)
    rs = ""
    while i<l:
        c = s[i]
        i+=1
        if c == '\\':
            x = s[i]
            i+=1
            if x == 'u':
                v = int(s[i:i+4], 16)
                r = chr(v)
                rs+=r
                i+=4
                continue
            elif x in maps:
                r = maps[x]
                rs+=r
                continue
        rs+=c
    return rs

pass
bl2,bu = b'\\u'
def ubytes(s, code = "utf-8"):
    i = 0
    l=len(s)
    rs = b""
    while i<l:
        c = s[i]
        i+=1
        if c == bl2:
            x = s[i]
            i+=1
            if x == bu:
                v = int(s[i:i+4], 16)
                r = chr(v).encode(code)
                rs+=r
                i+=4
                continue
            elif x in bmaps:
                r = bmaps[x]
                rs+=r
                continue
        rs+=bytes([c])
    return rs

pass
def ub2s(s, code = "utf-8"):
    return ubytes(s, code).decode(code)

pass


#coding=utf-8
from buildz import xf, argx, pyz, ioc
from ..test import Help as BaseHelp
from os.path import dirname, join
from ..test import text_sfx, args, maps
dp = dirname(dirname(__file__))
default_fp = join(dp, "res")
class Help(BaseHelp):
    def __init__(self, dp, fp):
        fp = join(dp, fp)
        self.text = xf.loads(xf.fread(fp))['text'+text_sfx]
        self.text = self.text.replace("{default}", default_fp)
    def deal(self):
        print(self.text)

pass



from .readz import loads, load, loadf, loads_args, loadx, is_args, loadxf
from .write import dumps, dump, dumpf, dumpx, dumpxf
from .file import *
from .mapz import *
from .xargs import fetch as args
try:
    # C加速代码
    from cxf import loads
except:
    pass
pass
__author__ = "Zzz, emails: 1174534295@qq.com, 1309458652@qq.com"




from .loader import mg, buffer, base

from .loader.deal import nextz, spt, strz, listz, spc, setz, mapz, reval, lrval
def bl(val):
    trues = [base.BaseDeal.like(k, val) for k in ["true", "True", "1"]]
    trues += [1,True]
    falses = [base.BaseDeal.like(k, val) for k in ["false", "False", "0"]]
    falses += [0,False]
    if val in trues:
        return True
    elif val in falses:
        return False
    else:
        raise Exception("unknown bool val")

pass
def build_lrval(mgs):
    fcs = lrval.Fcs()
    fcs.set("float", float)
    fcs.set("f", float)
    fcs.set("int", int)
    fcs.set("i", int)
    fcs.set("bool", bl)
    fcs.set("bl", bl)
    fcs.set("null", lambda x:None)
    fcs.set("nil", lambda x:None)
    fcs.set("n", lambda x:None)
    mgs.add(lrval.LRValDeal("<",">",fcs))

pass
def build_val(mgs):
    mgs.add(reval.ValDeal("[\+\-]?\d+", int))
    mgs.add(reval.ValDeal("[\+\-]?\d*\.\d+", float))
    mgs.add(reval.ValDeal("[\+\-]?\d+e[\+\-]?\d+", float))
    mgs.add(reval.ValDeal("null", lambda x:None))
    mgs.add(reval.ValDeal("true", lambda x:True))
    mgs.add(reval.ValDeal("false", lambda x:False))

pass
def build():
    mgs = mg.Manager()
    mgs.add(spc.PrevSpcDeal())
    build_val(mgs)
    mgs.add(strz.PrevStrDeal("r'''","'''",0,0,0))
    mgs.add(strz.PrevStrDeal('r"""','"""',0,0,0))
    mgs.add(strz.PrevStrDeal("r'","'",1,0,0))
    mgs.add(strz.PrevStrDeal('r"','"',1,0,0))
    mgs.add(strz.PrevStrDeal("###","###",0,1))
    mgs.add(strz.PrevStrDeal("/*","*/",0,1))
    mgs.add(strz.PrevStrDeal("'''","'''",0,0,1))
    mgs.add(strz.PrevStrDeal('"""','"""',0,0,1))
    mgs.add(strz.PrevStrDeal("#","\n",1,1))
    mgs.add(strz.PrevStrDeal("//","\n",1,1))
    mgs.add(strz.PrevStrDeal("'","'",1,0,1))
    mgs.add(strz.PrevStrDeal('"','"',1,0,1))
    mgs.add(setz.SetDeal(':'))
    mgs.add(setz.SetDeal('='))
    mgs.add(spt.PrevSptDeal(',',1))
    mgs.add(spt.PrevSptDeal(';',1))
    mgs.add(spt.PrevSptDeal('\n'))
    build_lrval(mgs)
    mgs.add(listz.ListDeal("(", ")"))
    mgs.add(listz.ListDeal("[", "]"))
    mgs.add(mapz.MapDeal("{", "}"))
    mgs.add(nextz.PrevNextDeal())
    return mgs

pass
def load(read):
    mgs = build()
    return msg.load(read)
def loads(s):
    # lr = "{}"
    # ls = "{[("
    # if type(s)==bytes:
    #     lr = lr.encode()
    #     ls = ls.encode()
    # x = s.strip()
    # if len(x)>0 and x[0] not in ls:
    #     s = lr[0]+s+lr[1]
    mgs = build()
    input = buffer.BufferInput(s)
    return mgs.load(s)

pass




class BaseDeal:
    def has_prev(self):
        return 1
    def has_deal(self):
        return 1
    def sp(self):
        return super(self.__class__, self)
    def regist(self, mgs):
        if self._id is None:
            self._id = mgs.regist()
    def id(self):
        return self._id
    def same(self, s, target):
        return self.like(s, target) == target
    @staticmethod
    def like(s, target):
        if type(target) ==type(s):
            return s
        if type(s)==str:
            return s.encode()
        return s.decode()
    def __str__(self):
        return "BaseDeal"
    def __repr__(self):
        return str(self)
    def __init__(self, *argv, **maps):
        self._id = None
        self.init(*argv, **maps)
    def init(self, *argv, **maps):
        pass
    def prev(self, buffer, queue, pos):
        """
            input:
                buffer: 1,读取字符串/字节，没有pop的时候，会保留读取的数据据, buffer.read(size=1, pop = False), buffer.pop_read(size=1)
                        2,缓存, buffer.add(chars), buffer.lget(size=1): chars, buffer.rget(size=1): chars
                        buffer.lpop(size=1):None, buffer.rpop(size=1)
                queue: 存放生成节点的队列, queue=list<obj>
                pos: 位置计算: pos.update(chars), pos.get(index=0|index<=0): return [row, column]
            output:
                bool: 是否进行了处理
        """
        return False
    def deal(self, queue, stack):
        """
            input:
                queue: 节点输入队列
                stack: 存放生成结果的栈
            output:
                bool: 是否进行了处理
        """
        return False

pass




from .. import xf
from ..base import Base
import re

def default_update(node, args, up):
    """
        默认args是patterns, deep形式
    """
    deep = args[1]
    if deep == 0:
        return None
    if deep>0:
        deep -= 1
    return [args[0], deep]

pass
class DefaultSearch:
    """
        默认args是patterns, deep形式
        patterns:
        [or, [re, key, 'a.*'], ['=', key, ]]
    """
    def __init__(self):
        self.opts = {}
        self.opts['or'] = self.fc_or
        self.opts['and'] = self.fc_and
        self.opts['not'] = self.fc_not
        self.opts['xor'] = self.fc_xor
        self.sets(("=", "eq"), self.make_logic(lambda data, val:data==val))
        self.sets((">", "bg"), self.make_logic(lambda data, val:data>val))
        self.sets((">=", "be"), self.make_logic(lambda data, val:data>=val))
        self.sets(("<", "lt"), self.make_logic(lambda data, val:data<val))
        self.sets(("<=", "le"), self.make_logic(lambda data, val:data<=val))
        self.sets("in", self.make_logic(lambda data, val:data in val))
        self.sets("re", self.make_logic(lambda data, val:len(re.findall(val, str(data)))>0))
        self.sets(["$", "eval"], self.fc_eval)
        self.sets(["*", "exec"], self.fc_exec)
        self.sets("mkv", self.fc_mkv)
    def fc_eval(self, node, args, curr):
        cmd = args[0]
        return eval(cmd)
    def fc_exec(self, node, args, curr):
        """
            exec(cmd)
            return self.rst
        """
        cmd = args[0]
        exec(cmd)
        return self.rst
    def sets(self, keys, fc):
        if type(keys) not in (list, tuple):
            keys = [keys]
        for k in keys:
            self.opts[k] = fc
    def fc_mkv(self, node, args, curr):
        data = node.data
        if type(data)!=dict:
            return False
        key = args[0]
        val = args[1]
        if key not in data:
            return False
        return data[key].to_data()==val
    @staticmethod
    def fetch_judge_by(judge_by, node, curr):
        #print(f"[TESTZ] fetch_judge_by: {judge_by}, {node}, {curr}")
        if judge_by in ('key', 'k'):
            if curr is None:
                return None, False
            key = curr[0]
            return key,True
        elif judge_by in ('index', 'i'):
            if curr is None:
                return None, False
            key = curr[0]
            return int(key),True
        elif judge_by in ('value', 'val', 'v'):
            return node.to_data(),True
        else:
            assert False, f"unsupport judge_by: {judge_by}"
    def make_logic(self, fc):
        def check(node, args, curr):
            judge_by = args[0]
            data, get = self.fetch_judge_by(judge_by, node, curr)
            if not get:
                return False
            val = args[1]
            return fc(data, val)
        return check
    def fc_xor(self, node, args, curr):
        a = self.fc(node, args[0], curr)
        b = self.fc(node, args[1], curr)
        return a^b
    def fc_not(self, node, args, curr):
        return not self.fc(node, args[0], curr)
    def fc_or(self, node, args, curr):
        for pattern in args:
            if self.fc(node, pattern, curr):
                return True
        return False
    def fc_and(self, node, args, curr):
        for pattern in args:
            if not self.fc(node, pattern, curr):
                return False
        return True
    def fc(self, node, args, curr):
        opt = args[0]
        #print(f"[TESTZ] opt: {opt}, {type(opt)}, args: {args}")
        assert opt in self.opts, f"unkonwn opt {opt} in {list(self.opts.keys())}"
        return self.opts[opt](node, args[1:], curr)
    def __call__(self, node, args, curr):
        args = args[0]
        jg = self.fc(node, args, curr)
        if not jg:
            return []
        return [[curr, node]]

pass
default_search = DefaultSearch()
class SearchArray(Base):
    @staticmethod
    def parse(data):
        return SearchArray.single(Node.parse(data))
    @staticmethod
    def single(data):
        return SearchArray([[None, data]])
    def str(self):
        return str(self.arr)
    def init(self, arr):
        self.arr = arr
        self.size = len(arr)
    def expand(self, ks):
        if ks is None:
            return []
        return self.expand(ks[1]) + [ks[0]]
    def datas(self):
        rst = [[self.expand(ks), nd.to_data()] for ks, nd in self.arr]
        return rst
    def vals(self):
        rst = [nd.to_data() for ks, nd in self.arr]
        return rst
    def keys(self):
        rst = [self.expand(ks) for ks, nd in self.arr]
        return rst
    def searchs(self, args):
        """
            array.searchs((patterns, fc_search, fc_update), (patterns, fc_search, fc_update), ...)
        """
        rst = self.arr
        for patterns, fc_search, fc_update in args:
            tmp = []
            for ks, node in rst:
                tmp += node.search(patterns, fc_search, fc_update, ks)
            rst = tmp
        return SearchArray(rst)
    def searchs_fc(self, args, fc_search=None, fc_update=None):
        rst = self.arr
        if type(args)==str:
            args = xf.loads(args)
        for patterns in args:
            tmp = []
            for ks, node in rst:
                tmp += node.search(patterns, fc_search, fc_update, ks)
            rst = tmp
        return SearchArray(rst)
    def searchs_dfc(self, args, deep=-1, fc_search=None, fc_update=None):
        rst = self.arr
        if type(args)==str:
            args = xf.loads(args)
        for patterns in args:
            tmp = []
            for ks, node in rst:
                tmp += node.search([patterns,deep], fc_search, fc_update, ks)
            rst = tmp
        return SearchArray(rst)

pass
class Node:
    @staticmethod
    def parse(data):
        if type(data)==list:
            rst = [Node.parse(it) for it in data]
        elif type(data)==dict:
            rst = {k:Node.parse(v) for k,v in data.items()}
        else:
            rst = data
        return Node(rst)
    def to_data(self):
        if type(self.data)==list:
            rst = [it.to_data() for it in self.data]
        elif type(self.data)==dict:
            rst = {k:v.to_data() for k,v in self.data.items()}
        else:
            rst = self.data
        return rst
    def __str__(self):
        return xf.dumps(self.to_data())
    def __repr__(self):
        return self.__str__()
    def __init__(self, data):
        self.data = data
    def search(self, args, fc_search=None, fc_update = None, curr = None):
        """
            args: 匹配条件
            返回:
                SearchArray(rst)
                rst: [
                    [keys, val],
                    ...
                ]
            SearchArray(node).searchs(...)
        """
        if fc_update is None:
            fc_update = default_update
        if fc_search is None:
            fc_search = default_search
        args = fc_update(self, args, curr)
        if args is None:
            return []
        rst = fc_search(self, args, curr)
        if type(self.data)==list:
            for i in range(len(self.data)):
                it = self.data[i]
                crr = [i, curr]
                rst += it.search(args, fc_search, fc_update, crr)
        elif type(self.data)==dict:
            for k,v in self.data.items():
                crr = [k, curr]
                rst += v.search(args, fc_search, fc_update, crr)
        return rst

pass



#
from ..tools import *
from buildz.ioc import wrap
@wrap.obj(id="defs")
@wrap.obj_args("ref, cache", "ref, log", "ref, cache.modify")
class Defs(Base):
    def init(self, cache, log, upd):
        self.cache = cache
        self.log = log.tag("Defs")
        self.upd = upd
    def update(self, s, defs, ignore=None):
        if type(s)==dict:
            rst = {}
            for k,v in s.items():
                if k == ignore:
                    continue
                k,v = self.update(k, defs), self.update(v, defs)
                rst[k] = v
            return rst
        elif type(s)==list:
            rst = []
            for v in s:
                v = self.update(v,defs)
                rst.append(v)
            return rst
        for k,v in defs.items():
            if k==s:
                s = v
                continue
            if type(k)!=str:
                continue
            if type(s)!=str:
                continue
            s = s.replace(k,str(v))
        return s
    def call(self, data, fc=None):
        defs = self.upd(xf.g(data, defs = {}))
        data = self.update(data, defs, "defs")
        data = self.upd(data)
        if fc is None:
            return True
        return fc(data)

pass





import time
from ... import Base
from ...logz import FpLog
class Clock(Base):
    def default(self, cost, fc, rst, *a, **b):
        print(f"cost {cost} sec on {fc}")
    def init(self, fc=None):
        if fc is None:
            fc = self.default
        self.fc = fc
    def call(self, fc):
        curr = time.time()
        def tfc(*a,**b):
            curr = time.time()
            rst = fc(*a,**b)
            cost = time.time()-curr
            self.fc(cost, fc, rst, *a, **b)
            return rst
        return tfc

pass
clock = Clock
timecost = Clock
showcost = Clock()



import sqlite3
import sys,os
from .basez import SimpleDv, fetch
from .structz import CMD
from buildz import xf,fz
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        rows = self.cursor.description
        keys = [k[0].lower() for k in rows]
        result = []
        result.append(keys)
        if rst is None or len(rst)==0:
            return result
        result += rst
        return result
    def new_con(self):
        return sqlite3.connect(self.fp)
    def new_cursor(self):
        return self.con.cursor()
    def __init__(self, fp, as_map=False, *argv, **maps):
        #def __init__(self, fp):
        self.con = None
        self.cursor = None
        self.as_map = as_map
        self.init(fp)
    def init(self, fp):
        fz.makefdir(fp)
        self.fp = fp
    def sql_tables(self, table = None):
        """
            if not note, use name instead
            require:
                table_name, table_note
        """
        query_table = ""
        if table is not None:
            query_table =  f" and name = '{table}'"
        return f"select name as table_name, '' as table_note from sqlite_master where type='table' {query_table};"
    def sql_indexes(self, table=None):
        """
            require:
                table_name, index_name, is_unique, index_type, index_note
        """
        query_table = ""
        if table is not None:
            query_table =  f" and tbl_name = '{table}'"
        return f"select tbl_name as table_name, name as index_name, -1 as is_unique, '?' as index_type, '' as index_note from sqlite_master where type='index' {query_table};"
    def columns(self, table, as_map=None):
        """
            require:
                table_name, column_name, column_type, column_default, nullable, column_offset, column_note
        """
        rst = self.query(f"PRAGMA table_info({table})", as_map=0)
        dts = rst[1:]
        keys = "table_name, column_name, column_type, column_default, nullable, column_offset, column_note".split(", ")
        rst = [keys]
        for dt in dts:
            cid, name, _type, notnull, dflt_value, pk = dt
            tmp = [table, name, _type, dflt_value, notnull, cid, '']
            rst.append(tmp)
        rst = self.out_list_sqlite3(rst, as_map)
        return rst
    def index_keys(self, index, as_map=None):
        """
            require:
                index_name, column_name, index_offset, column_note
        """
        rst = self.query(f"PRAGMA index_info({index})", as_map=0)
        dts = rst[1:]
        keys = "index_name, column_name, index_offset, column_note".split(", ")
        rst = [keys]
        for dt in dts:
            seqno, cid, name = dt
            tmp = [index, name, seqno, '']
            rst.append(tmp)
        return self.out_list_sqlite3(rst, as_map)
    def out_list_sqlite3(self, query_result, as_map=None):
        if as_map is None:
            as_map = self.as_map
        rst = query_result
        if as_map and len(rst)>0:
            if len(rst)==1:
                return []
            keys = rst[0]
            rst = rst[1:]
            rst = [{k:v for k,v in zip(keys, dt)} for dt in rst]
        return rst

pass
def build(argv, conf):
    root = xf.g(conf, root=None)
    fp = argv[0]
    if root is not None:
        fp = os.path.join(root, fp)
    as_map = xf.g(conf, as_map=False)
    dv = Db(fp, as_map)
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass



#coding=utf-8
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class ListDeal(FormatDeal):
    """
        list:
            {
                id: id
                type: list
                data: [
                    item_conf,
                    ...
                ]
            }
        简写:
            [[id, list], data]
            [list, data]
        例:
            [list, [[ref, obj.test], [env, path]]] // 返回列表,第0个元素是对数据obj.test的索引，第二个是环境变量path的值
    """
    def init(self, fp_lists=None, fp_defaults=None):
        super().init("ListDeal", fp_lists, fp_defaults, join(dp, "conf", "list_lists.js"), None)
    def deal(self, edata:EncapeData):
        data = edata.data
        data = self.fill(data)
        lists = xf.g1(data, list = [], data=[])
        conf = edata.conf
        rst = [self.get_obj(k, conf, edata.src, edata.info) for k in lists]
        return rst

pass
